#ifndef kfgnKJNxcmep2w8fnmxow238cmsh48FRJ
#define kfgnKJNxcmep2w8fnmxow238cmsh48FRJ
//Copyright (c) by Matthias Benkmann
//permission to use this code granted for commercial and uncommercial projects.

//mail questions and comments to mbenkmann@gmx.de

//bitwiseCRC is a class that is able to calculate CRCs of any length up to
//32bits for a stream of bits of arbitrary length (does NOT need to be
//multiple of 8 long).
//The class automatically assumes that the bit stream
//is being followed by a number of bits equal to the desired CRC length.
//This means that when using the class to calculate the CRC for a
//data stream you do NOT transmit these 0-bits.
//If you want to verify whether a data stream and a CRC are consistent,
//you transmit the data stream followed by the CRC. The CRC returned for
//this bundle must be 0
//Example: You want to calculate a 16bit CRC for the byte sequence
//AB CD 00 00, where the 00 00 is the placeholder
//where the CRC will be filled in later. The only data you transmit to
//the bitwiseCRC object are the bytes AB and CD. The trailing 16bit 0
//is automatically assumed so that after passing the last bit of CD to
//the object, the desired 16bit CRC is already available and would only
//be destroyed if you transmitted the 00 00, too.
//Let's say the CRC that comes out is 1234 (all numbers in this example are
//hex). Now you put this into the
//data stream (LSB-first!!!) where the 00 00 was.
//Now you have AB CD 34 12
//You transmit all these bytes to the CRC object (after having reset it)
//and you get 0, because the 12 34 is consistent with the data bytes AB CD
//and 0 signifies consistency.


//!!!!!!ATTENTION!!!!!!!  Every bitwiseCRC object has its !!!!!!!!!!!!!!
//!!!!!!!!!!!!!!!!!!!!!!!! own table of 256 unsigned longs !!!!!!!!!!!!!
//!!!!!!!!!!!!!!!!!!!!!!!  to speed up calculation !!!!!!!!!!!!!!!!!!!!
//!!!!!!!!!!!!!!!!!!!!!! So don't have too many of them !!!!!!
//constructor description: If you do NOT pass a value to the constructor
//                         then the CRC calculated will be a 32Bit CRC
//                         for the CRC polynomial
//    x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x^1+x^0
//                        which is the same polynomial as suggested in
//                        ISO 3309
//                 If you DO pass a value to the constructor it will
//                 be interpreted as a CRC polynomial with MSB dropped and
//                 whose binary digits are in reverse order (i.e. MSB in LSB).
//                 Example:
//                 You want to tell the CRC algorithm to use the above
//                 ISO 3309 polynomial. Its binary representation is
//                 1 00000100 11000001 00011101 10110111
//                 The MSB is dropped, so we get
//                   00000100 11000001 00011101 10110111
//                 The binary digits must be written backwards, so we get
//                   11101101 10111000 10000011 00100000
//                 or in hex
//                   ED       B8       83       20
//                 which is the value you must pass to the constructor
//                 NOTE: Don't worry about telling the object the length
//                 of the CRC polynomial. Knowing this length is not
//                 necessary for CALCULATING the CRC. Only the caller
//                 needs to know it, in order to interpret the result.
//             ATTENTION!!! 0 is not a valid parameter for the
//                 constructor. If you pass 0 the the results will
//                 be undefined.
//Use following methods to access:
//  Init(Poly(opt.)): resets the object for the calculation of the CRC
//                    for a new data stream (NEVER FORGET TO CALL INIT BEFORE
//                    CRCing a new data stream). If the optional parameter
//                    Poly is passed and Poly!=0 then the CRC polynomial
//                    used will be changed to the new value.
//                    NOTE: Poly must be passed in the same format
//                    as it is passed to the constructor (see above)
//                ATTENTION!!! If Poly=0 or not passed at all then
//                    the object will be reset, but the polynomial will not
//                    be changed.
//  Reset():  the same as Init() without the optional parameter
//  ProcessBit(Bit): will chew the next bit of the data stream.
//                   Note: You should send bytes LSB-first to be consistent
//                   with ProcessByte
//  ProcessByte(Byte): will chew the 8 bits of Byte LSB-first.
//                     !!!!This function should be used whenever possible.
//                     It uses a look-up table to speed up processing. It
//                     is a lot faster than pushing the 8 bits through
//                     ProcessBit().!!!!
//  (unsigned long): When cast into an unsigned long then the object
//                   will return the CRC for the data stream that was
//                   already processed.
//                   ATTENTION!! The CRC returned is reversed, which means
//                   that the LSB of the value returned by the cast op,
//                   is in reality the MSB of the CRC you want. This
//                   is actually very convenient, since this means that
//                   the CRC you want will always be stored in the lower
//                   bits. So if you want for instance a 16-bit CRC, you
//                   can use ("word" means a 16bit unsigned int here)
//                             word CRC16=CRCObject;
//                   instead of
//                             word CRC16=(unsigned long)CRCObject shr 16;
//                   as you would have to if CRCObject returned the CRC
//                   UNreversed.
//                   NOTE: The CRC used by .PNG files is the same as returned
//                   by bitwiseCRC objects with the default polynomial,
//                   EXCEPT that the .PNG CRC is (for obscure reasons)
//                   binary complemented compared to bitwiseCRC.
//                   Note: I got this information from some .PNG example code
//                   I have not verified it. Please tell me if it's accurate.
#include <alloc.h>
#include "stdmacs.h"

class StdMemErr{};  //thrown in case of memory error

class bitwiseCRC
begin
  unsigned long CRCPoly;
  unsigned long CRC;
  unsigned long* CRC_table;
public:
  operator unsigned long() const {return(CRC);};
  bitwiseCRC& ProcessBit(unsigned Bit);
  bitwiseCRC& ProcessByte(unsigned Byte);
  bitwiseCRC& Init(unsigned long Poly=0);
  bitwiseCRC& Reset() {return(Init());};
  bitwiseCRC(unsigned long Poly=0xEDB88320);
  ~bitwiseCRC();
end;

#endif
